#include <TGUI/Config.hpp>
#include <TGUI/Backend/SFML-Graphics.hpp>
#include <TGUI/Timer.hpp>

#include "GuiExSFMLGraphics.hpp"

#include <thread>

namespace tgui
{
    void SFML_GRAPHICS::GuiEx::mainLoop(Color clearColor)
    {
        if (!m_window)
            return;

        m_backendRenderTarget->setClearColor(clearColor);

        // Helper function that calculates the amount of time to sleep, which is 10ms unless a timer will expire before this time
        const auto getTimerWakeUpTime = []{
            Optional<Duration> duration = Timer::getNextScheduledTime();
            if (duration && (*duration < std::chrono::milliseconds(10)))
                return *duration;
            else
                return Duration{std::chrono::milliseconds(10)};
        };

        setDrawingUpdatesTime(false);

        sf::Event event;
        bool refreshRequired = true;
        std::chrono::steady_clock::time_point lastRenderTime;
        bool windowOpen = m_window->isOpen();
        while (m_window->isOpen()) // Don't just check windowOpen, user code can also call window.close()
        {
            bool eventProcessed = false;
            while (true)
            {
                while (m_window->pollEvent(event))
                {
                    if (handleEvent(event))
                        eventProcessed = true;

                    if (event.type == sf::Event::Closed || m_toBeClosed)
                    {
                        // We don't call m_window->close() as it would destroy the OpenGL context, which will cause messages to be
                        // printed in the terminal later when we try to destroy our backend renderer (which tries to clean up OpenGL resources).
                        // The close function will be called by the window destructor.
                        windowOpen = false;
                        eventProcessed = true;
                    }
                    else if (event.type == sf::Event::Resized)
                    {
                        eventProcessed = true;
                    }
                }

                if (updateTime())
                    break;

                if (eventProcessed || refreshRequired)
                    break;

                std::this_thread::sleep_for(std::chrono::nanoseconds(getTimerWakeUpTime()));
            }

            if (!windowOpen)
                break;

            refreshRequired = true;

            // Don't try to render too often, even when the screen is changing (e.g. during animation)
            const auto timePointNow = std::chrono::steady_clock::now();
            const auto timePointNextAllowed = lastRenderTime + std::chrono::milliseconds(20);
            if (timePointNextAllowed > timePointNow)
            {
                const auto timerWakeUpTime = getTimerWakeUpTime();
                if (timePointNextAllowed - timePointNow < timerWakeUpTime)
                    std::this_thread::sleep_for(timePointNextAllowed - timePointNow);
                else
                    std::this_thread::sleep_for(std::chrono::nanoseconds(timerWakeUpTime));

                continue;
            }

            m_backendRenderTarget->clearScreen();
            draw();
            m_window->display();

            refreshRequired = false;
            lastRenderTime = std::chrono::steady_clock::now(); // Don't use timePointNow to provide enough rest on low-end hardware
        }
    }
}
