/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// TGUIEX - Texus' Graphical User Interface Extensions
// Copyright (C) 2022 Steffen Ploetz (steffen.ploetz@cityweb.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Converter: https://notisrac.github.io/FileToCArray/

// Required includes: <cstdint>

#ifndef TGUIEX_IMAGEBYTES_HPP
#define TGUIEX_IMAGEBYTES_HPP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief An excerpt of tango icon theme icons
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
namespace tangoIconTheme
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief A very simple class to hold icon data
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    class Icon
    {
    private:
        const std::uint8_t* bytes = 0;    //!< The icon bytes
        std::size_t byteCount = 0; //!< The number of icon bytes
        std::size_t width = 0;     //!< The icon width in pixels
        std::size_t height = 0;    //!< The icon height in pixels

    public:
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Created an icon
        /// @param bytes      The icon bytes
        /// @param byteCount  The number of icon bytes
        /// @param width      The icon width in pixels
        /// @param height     The icon height in pixels
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        Icon(const std::uint8_t* bytes, std::size_t byteCount, std::size_t width, std::size_t height)
        {
            this->bytes = bytes;
            this->byteCount = byteCount;
            this->width = width;
            this->height = height;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Gets the icon bytes
        ///
        /// @return The icon bytes
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        const std::uint8_t* getBytes() const
        {
            return bytes;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Gets the icon width in pixels
        ///
        /// @return The icon width in pixels
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        const std::size_t getWidth() const
        {
            return width;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Gets the icon height in pixels
        ///
        /// @return The icon height in pixels
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        const std::size_t getHeight() const
        {
            return height;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Gets the number of icon bytes
        ///
        /// @return The number of icon bytes
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        const std::size_t getByteCount() const
        {
            return byteCount;
        }
    };

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Provide 22px x 22px icons
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    class Icons22
    {
        public:
            static Icon Action_DocumentOpen; //!< The action icon: document open
    };

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Provide 28px x 28px icons, derived from 22px x 22px icons (with 3px padding added)
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    class Icons28
    {
        public:
            static Icon Action_DocumentOpen; //!< The action icon: document open
            static Icon Action_DocumentSave; //!< The action icon: document save
            static Icon Action_EditCopy; //!< The action icon: edit copy
            static Icon Action_EditPaste; //!< The action icon: edit paste
            static Icon Action_SystemLogOut; //!< The action icon: system log-out

            static Icon Apps_HelpBrowser; //!< The apps icon: help browser
            static Icon Apps_InfoBrowser; //!< The apps icon: info browser
    };
}

#endif // TGUIEX_IMAGEBYTES_HPP
